#include	"sq.h"

#include	<stdio.h>
#include	<time.h>
#include	<sys/time.h>
#include	<sys/param.h>


/*** Variables -- For logging ***/
extern FILE * outlog;
extern FILE * errlog;


/*** Variables -- Imported from Virtual Machine ***/
extern unsigned char	*memory;
extern int interruptKeycode;
extern int interruptPending;
extern int interruptCheckCounter;

/*** Crash debug -- Imported from Virtual Machine ***/
extern const int primitiveIndex;
int getCurrentBytecode(void);
int printCallStack(void);

/*** Variables -- image and path names ***/
#define IMAGE_NAME_SIZE	MAXPATHLEN
char imageName[MAXPATHLEN+1];		/* full path to image */
char shortImageName[MAXPATHLEN+1];	/* just the base name */
char vmPath[MAXPATHLEN+1];			/* full path to interpreter's directory */
char *displayName= 0;				/* name of display, which in DOS is a noop */

int initialArgc;
char ** initialArgv;

/***** Palette Constants ******/
#define UNIX_MAX_PIXEL 65535
#define ALLEGRO_MAX 63
PALETTE stPal;

int savedWindowSize= 0;	/* initial size of window */
int initialHeapSize= 5*1024*1024;
BITMAP * current_cursor;	/* current cursor for the display */

/*** Variables -- Event Recording ***/
#define KEYBUF_SIZE 64

int keyBuf[KEYBUF_SIZE];	/* circular buffer */
int keyBufGet=	0;			/* index of next item of keyBuf to read */
int keyBufPut=	0;			/* index of next item of keyBuf to write */
int keyBufOverflows=	0;	/* number of characters dropped */

int buttonState= 0;		/* mouse button and modifier state when mouse
                              button went down or 0 if not pressed */

struct timeval startUpTime;
#define ITIMER_INIT_SECS 31536000	/* 1 year of millisecond clock */
/* maximum input polling frequencey */
#define MAXPOLLSPERSEC   33

/*** VM Home Directory Path ***/
int vmPathSize(void)
	{
	return strlen(vmPath);
	}

int vmPathGetLength(int sqVMPathIndex, int	length) 
	{
   	char *stVMPath= (char *)sqVMPathIndex;
	int count, i;

   count = strlen(vmPath);
   count = (length <	count) ?	length :	count;

   /* copy the file name into the Squeak string */
   for (i= 0; i < count; i++)
       stVMPath[i]= vmPath[i];

   return count;}

/*** DOS related variables ***/
BITMAP *memory_bitmap;
int stWidth, stHeight;			/* the width and height of the dos screen */

extern int convertToSqueakTime(int);

/* Pixmap Colors */
void SetColorEntry(int index, float r, float g, float b) 
{/* This code is written to leverage the unix implementation.
Convert from unix RGB values to allegro values */
stPal[index].r = r/UNIX_MAX_PIXEL * ALLEGRO_MAX;
stPal[index].g = g/UNIX_MAX_PIXEL * ALLEGRO_MAX;
stPal[index].b = b/UNIX_MAX_PIXEL * ALLEGRO_MAX;
}



SetUpPixmaps() {
 /* 1-bit colors (monochrome) */
  SetColorEntry(0, 65535, 65535, 65535);	/* white or transparent */
  SetColorEntry(1,     0,     0,     0);	/* black */

  /* additional colors for 2-bit color */
  SetColorEntry(2, 65535, 65535, 65535);	/* opaque white */
  SetColorEntry(3, 32768, 32768, 32768);	/* 1/2 gray */

  /* additional colors for 4-bit color */
  SetColorEntry( 4, 65535,     0,     0);	/* red */
  SetColorEntry( 5,     0, 65535,     0);	/* green */
  SetColorEntry( 6,     0,     0, 65535);	/* blue */
  SetColorEntry( 7,     0, 65535, 65535);	/* cyan */
  SetColorEntry( 8, 65535, 65535,     0);	/* yellow */
  SetColorEntry( 9, 65535,     0, 65535);	/* magenta */
  SetColorEntry(10,  8192,  8192,  8192);	/* 1/8 gray */
  SetColorEntry(11, 16384, 16384, 16384);	/* 2/8 gray */
  SetColorEntry(12, 24576, 24576, 24576);	/* 3/8 gray */
  SetColorEntry(13, 40959, 40959, 40959);	/* 5/8 gray */
  SetColorEntry(14, 49151, 49151, 49151);	/* 6/8 gray */
  SetColorEntry(15, 57343, 57343, 57343);	/* 7/8 gray */


  /* additional colors for 8-bit color */
  /* 24 more shades of gray (does not repeat 1/8th increments) */
  SetColorEntry(16,  2048,  2048,  2048);	/*  1/32 gray */
  SetColorEntry(17,  4096,  4096,  4096);	/*  2/32 gray */
  SetColorEntry(18,  6144,  6144,  6144);	/*  3/32 gray */
  SetColorEntry(19, 10240, 10240, 10240);	/*  5/32 gray */
  SetColorEntry(20, 12288, 12288, 12288);	/*  6/32 gray */
  SetColorEntry(21, 14336, 14336, 14336);	/*  7/32 gray */
  SetColorEntry(22, 18432, 18432, 18432);	/*  9/32 gray */
  SetColorEntry(23, 20480, 20480, 20480);	/* 10/32 gray */
  SetColorEntry(24, 22528, 22528, 22528);	/* 11/32 gray */
  SetColorEntry(25, 26624, 26624, 26624);	/* 13/32 gray */
  SetColorEntry(26, 28672, 28672, 28672);	/* 14/32 gray */
  SetColorEntry(27, 30720, 30720, 30720);	/* 15/32 gray */
  SetColorEntry(28, 34815, 34815, 34815);	/* 17/32 gray */
  SetColorEntry(29, 36863, 36863, 36863);	/* 18/32 gray */
  SetColorEntry(30, 38911, 38911, 38911);	/* 19/32 gray */
  SetColorEntry(31, 43007, 43007, 43007);	/* 21/32 gray */
  SetColorEntry(32, 45055, 45055, 45055);	/* 22/32 gray */
  SetColorEntry(33, 47103, 47103, 47103);	/* 23/32 gray */
  SetColorEntry(34, 51199, 51199, 51199);	/* 25/32 gray */
  SetColorEntry(35, 53247, 53247, 53247);	/* 26/32 gray */
  SetColorEntry(36, 55295, 55295, 55295);	/* 27/32 gray */
  SetColorEntry(37, 59391, 59391, 59391);	/* 29/32 gray */
  SetColorEntry(38, 61439, 61439, 61439);	/* 30/32 gray */
  SetColorEntry(39, 63487, 63487, 63487);	/* 31/32 gray */

  /* The remainder of color table defines a color cube with six steps
     for each primary color. Note that the corners of this cube repeat
     previous colors, but simplifies the mapping between RGB colors and
     color map indices. This color cube spans indices 40 through 255.
     */
  {
    int r, g, b;

    for (r= 0; r < 6; r++)
      for (g= 0; g < 6; g++)
	for (b= 0; b < 6; b++)
	  {
	    int i= 40 + ((36 * r) + (6 * b) + g);
	    if (i > 255) error("index out of range in color table compuation");
	    SetColorEntry(i, (r * 65535) / 5, (g * 65535) / 5, (b * 65535) / 5);
	  }
  }

 set_palette (stPal);
}



/*** I/O Primitives ***/

int ioExit(void)
{
  exit(0);
  /* avoid the warnings here */
  return 0;
}


int ioBeep(void)
	{
	}


  
int ioGetButtonState(void)
	{
  	HandleEvents();
  	return buttonState;
	}



int ioGetKeystroke(void)
	{
 	 int keystate;

	  HandleEvents();
	  if (keyBufGet == keyBufPut)
  	  	return -1;  /* keystroke buffer is empty */

 	 keystate= keyBuf[keyBufGet];
 	 keyBufGet= (keyBufGet + 1) % KEYBUF_SIZE;
	 return keystate;
	}



int ioMSecs()
{
  struct timeval now;
  gettimeofday(&now, 0);
  if ((now.tv_usec -= startUpTime.tv_usec) < 0) {
    now.tv_usec += 1000000;
    now.tv_sec -= 1;
  }
  now.tv_sec -= startUpTime.tv_sec;
  return (now.tv_usec / 1000 + now.tv_sec * 1000) & 0x1fffffff;
}

int ioMicroMSecs()
	{
  	/* already have millisecond resolution */
  	return ioMSecs();
	}

int ioRelinquishProcessorForMicroseconds(int microSeconds)
	{
  	return microSeconds;
	}


int ioMousePoint(void)
{
     HandleEvents();
     return ((mouse_x << 16) | mouse_y);
}



int ioPeekKeystroke(void)
{
  int keystate;
  HandleEvents();
  if (keyBufGet == keyBufPut)
     return -1;  /* keystroke buffer is empty */

  keystate= keyBuf[keyBufGet];
  /* set modifer bits in buttonState to reflect the last keystroke fetched */
  //buttonState= ((keystate >> 5) & 0xF8) | (buttonState & 0x7);
  return keystate;
}


int ioProcessEvents(void)
{
  HandleEvents();
}



int ioScreenSize(void)
	{
 	/* w is high 16 bits; h is low 16 bits */
 	return ((SCREEN_W << 16) | SCREEN_H);
 	}


/* returns the local wall clock time */
int ioSeconds(void)
{
  extern int convertToSqueakTime(int unixTime);
  struct timeval tv;

  gettimeofday(&tv, 0);
#ifdef HAS_TIMEZONE
  return convertToSqueakTime(tv.tv_sec + (timezone * HAS_TIMEZONE));
#else
  return convertToSqueakTime(tv.tv_sec + localtime(&tv.tv_sec)->tm_gmtoff);
#endif
}




int ioSetCursor(int cursorBitsIndex, int offsetX, int offsetY)
{
	unsigned char data[32];	/* cursors are always 16x16 */
	int i, j;
	char *dp, aByte;
	unsigned int lineVal;

	show_mouse(NULL);
    for (i= 0; i < 16; i++)
    	{
//      	data[i*2+0]= ~(checkedLongAt(cursorBitsIndex + (4 * i)) >> 24) & 0xFF;
//      	data[i*2+1]= ~(checkedLongAt(cursorBitsIndex + (4 * i)) >> 16) & 0xFF;
      	data[i*2]= ((unsigned)checkedLongAt(cursorBitsIndex + (4 * i)) >> 24) & 0xFF;
      	data[i*2+1]= ((unsigned)checkedLongAt(cursorBitsIndex + (4 * i)) >> 16) & 0xFF;
     	}
    clear(current_cursor); 
	for (i=0;i<16;i++)
		{
			// For each bit set in data set the corresponding
			// byte in the current_cursor
			aByte = data[2*i];
			// Low byte
			if ((aByte & 128) == 128) current_cursor->line[i][0] = 1;
			if ((aByte & 64) == 64) current_cursor->line[i][1] = 1; 
			if ((aByte & 32) == 32) current_cursor->line[i][2] = 1; 
			if ((aByte & 16) == 16) current_cursor->line[i][3] = 1; 
			if ((aByte & 8) == 8) current_cursor->line[i][4] = 1; 
			if ((aByte & 4) == 4) current_cursor->line[i][5] = 1; 
			if ((aByte & 2) == 2) current_cursor->line[i][6] = 1; 
			if ((aByte & 1) == 1) current_cursor->line[i][7] = 1;
			// High Byte
			aByte = data[2*i+1];
			if ((aByte & 128) == 128) current_cursor->line[i][0+8] = 1;
			if ((aByte & 64) == 64) current_cursor->line[i][1+8] = 1; 
			if ((aByte & 32) == 32) current_cursor->line[i][2+8] = 1; 
			if ((aByte & 16) == 16) current_cursor->line[i][3+8] = 1; 
			if ((aByte & 8) == 8) current_cursor->line[i][4+8] = 1; 
			if ((aByte & 4) == 4) current_cursor->line[i][5+8] = 1; 
			if ((aByte & 2) == 2) current_cursor->line[i][6+8] = 1; 
			if ((aByte & 1) == 1) current_cursor->line[i][7+8] = 1;
		//memcpy(scaledBitmap->line[i], (char *) data+(2*i), 2);
		//scaledBitmap->line[i][0]=data[2*i];
		//scaledBitmap->line[i][1]=data[2*i+1];
		}
   /* select the custom cursor and set the focus point to the middle of it */
   set_mouse_sprite(current_cursor);
   set_mouse_sprite_focus(-offsetX, -offsetY);
   show_mouse(screen);
   return 1;
}




#define bytesPerLine(width, depth)	((((width)*(depth) + 31) >> 5) << 2)
#define bytesPerLineRD(width, depth)	((((width)*(depth)) >> 5) << 2)

int ioShowDisplay(int dispBitsIndex, int width, int height, int depth,
                        int affectedL, int affectedR, int affectedT, int affectedB)
{
int i=0;
char * tempBitmap=0;
int stBitsPerPixel=8;

if(affectedR < affectedL || affectedT > affectedB)
	return 1;
if (affectedR > width) affectedR= width;
if (affectedB > height) affectedB= height;
if (affectedL > width) affectedL= width;
if (affectedT > height) affectedT= height;
show_mouse(NULL);
copyReverseImageBytes(
	(int *)dispBitsIndex, 
	(int *)memory_bitmap->line[0],
	depth, width, height,
	affectedL, affectedT, affectedR, affectedB);
blit(memory_bitmap, 
	screen, 
	affectedL, 
	affectedT, 
	affectedL, 
	affectedT, 
	affectedR - affectedL, 
	affectedB - affectedT);
show_mouse(screen);
}


void SetUpWindow(char *displayName)
	{
	/* do nothing */
	}



void SetWindowSize(void)
{
  int width, height, maxWidth, maxHeight;

  if (savedWindowSize != 0)
    {
      width=  (unsigned)savedWindowSize >> 16;
      height= savedWindowSize & 0xFFFF;
    }
  else
    {
      width=  640;
      height= 480;
    }

  /* minimum size is 64 x 64 */
  width=  ( width > 64) ?   width : 64;
  height= (height > 64) ?  height : 64;

  /* maximum size is screen size */
  maxWidth=  stWidth;
  maxHeight= stHeight;
  width=  ( width <= maxWidth)  ?  width : maxWidth;
  height= (height <= maxHeight) ? height : maxHeight;
}



int getKeyboardModifiers() 
{
	int modifiers = 0;

	/* Set the high squeak bits
	Squeak Bits		<command>	<option>	<control>	<shift>
	DOS Equivalents	<alt>		<??????>	<control>	<shift><caps lock>
	*/			
	if (key_shifts & KB_ALT_FLAG)
		modifiers = modifiers | 1 << 6;
	if (key_shifts & KB_CTRL_FLAG)
		modifiers = modifiers | 1 << 4;
	if (key_shifts & (KB_SHIFT_FLAG | KB_CAPSLOCK_FLAG))
		modifiers = modifiers | 1 << 3;
	return modifiers;}


void setButtonState(void)
{
	int modifiers = 0;
	int whichButton = 0;

	modifiers = getKeyboardModifiers();
	/* Set button states appropriately */
	if (mouse_b & 1) 
		{
		if (key_shifts & KB_CTRL_FLAG)
			whichButton = 2; 		// yellow menu
		if (key_shifts & KB_ALT_FLAG)
			whichButton = 1;		// blue button
		else
			whichButton = 4;		// red button
		}
	if (mouse_b & 2)
		whichButton = 2;
	if (mouse_b & 4)
		whichButton = 1;
	buttonState = modifiers | whichButton;
}


int keyPressedWithAlt()
{/* Try to determine which key is being pressed with the ALT key.
	This check is needed since when ALT is pressed the keycode's 
	value is 0 */
	
	if (key[KEY_TILDE]) return 126;
	if (key[KEY_1]) return 49;
	if (key[KEY_2]) return 50;
	if (key[KEY_3]) return 51;
	if (key[KEY_4]) return 52;
	if (key[KEY_5]) return 53;
	if (key[KEY_6]) return 54;
	if (key[KEY_7]) return 55;
	if (key[KEY_8]) return 56;
	if (key[KEY_9]) return 57;
	if (key[KEY_0]) return 58;
	if (key[KEY_MINUS]) return 45;
	if (key[KEY_EQUALS]) return 61;
	if (key[KEY_Q]) return 113;
	if (key[KEY_W]) return 119;
	if (key[KEY_E]) return 101;
	if (key[KEY_R]) return 114;
	if (key[KEY_T]) return 116;
	if (key[KEY_Y]) return 121;
	if (key[KEY_U]) return 117;
	if (key[KEY_I]) return 105;
	if (key[KEY_O]) return 111;
	if (key[KEY_P]) return 112;
	if (key[KEY_OPENBRACE]) return 123; 
	if (key[KEY_CLOSEBRACE]) return 125;
	if (key[KEY_BACKSLASH]) return 92;
	if (key[KEY_A]) return 97;    
	if (key[KEY_S]) return 115;    
	if (key[KEY_D]) return 100;    
	if (key[KEY_F]) return 102;    
	if (key[KEY_G]) return 103;    
	if (key[KEY_H]) return 104;    
	if (key[KEY_J]) return 106;    
	if (key[KEY_K]) return 107;    
	if (key[KEY_L]) return 108;    
	if (key[KEY_COLON]) return 59;
	if (key[KEY_QUOTE]) return 39;
	if (key[KEY_Z]) return 122;
	if (key[KEY_X]) return 120;
	if (key[KEY_C]) return 99;    
	if (key[KEY_V]) return 118;    
	if (key[KEY_B]) return 98;    
	if (key[KEY_N]) return 110;    
	if (key[KEY_M]) return 109;    
	if (key[KEY_COMMA]) return 44;
	if (key[KEY_STOP]) return 46; 
	if (key[KEY_SLASH]) return 47;
}

int translateKey (int keycode) {

	int keystate = 0;
	int mods = 0;
	
	switch (keycode >> 8) {
		case KEY_BACKSPACE: keystate = 8; break;
		case KEY_DEL: keystate = 127; break;
		case KEY_INSERT: keystate = 5; break;
		case KEY_PGUP: keystate = 11; break;	// prior
		case KEY_PGDN: keystate = 12; break;	// next
		case KEY_END: keystate = 4; break;
		case KEY_HOME: keystate = 1; break;
		case KEY_LEFT: keystate = 28; break;
		case KEY_RIGHT: keystate = 29; break;
		case KEY_UP: keystate = 30; break;
		case KEY_DOWN: keystate = 31; break;
		default: keystate = keycode & 0xFF;
		}
	mods = getKeyboardModifiers();
	if (mods >= 64)
		{
		/* Check for another key press since when ALT is pressed
		the keycode's value is 0 */
		keystate = keyPressedWithAlt();
		}
	return (keystate | ((mods >> 3) << 8));
}	



void setKeyboardState(void)
{
   int keystate;
   /* Since readkey() blocks in Allegro check to see if a key is pressed.
   Do this until no keys are left in the buffer.  Add the keys in the
   buffer to the Smalltalk keyboard buffer */

   while (keypressed())
         {
		keystate = translateKey(readkey());
		if (keystate == interruptKeycode)
    			{
      		/* Note: interrupt key is "meta"; it not 
			reported as a keystroke */
      		interruptPending= true;
      		interruptCheckCounter= 0;
 			return;}

            keyBuf[keyBufPut]= keystate;
            keyBufPut = (keyBufPut + 1) % KEYBUF_SIZE;
            if (keyBufGet == keyBufPut)
            {
             /* buffer overflow; drop the last character */
             keyBufGet= (keyBufGet + 1) % KEYBUF_SIZE;
             keyBufOverflows++;
            }
         }
}

int HandleEvents(void) {
	setButtonState();     
	setKeyboardState();
	return false;
}




int copyReverseImageBytes(int *fromImageData, int *toImageData,
			  int depth, int width, int height,
			  int affectedL, int affectedT, int affectedR, int affectedB)
{
  register int scanLine, firstWord, lastWord;
  register int line;

  scanLine= bytesPerLine(width, depth);
  firstWord= scanLine*affectedT + bytesPerLineRD(affectedL, depth);
  lastWord= scanLine*affectedT + bytesPerLine(affectedR, depth);

  for (line= affectedT; line < affectedB; line++)
  {
    register unsigned char *from= (unsigned char *)((int)fromImageData+firstWord);
    register unsigned char *limit= (unsigned char *)((int)fromImageData+lastWord);
    register unsigned char *to= (unsigned char *)((int)toImageData+firstWord);
    while (from < limit)
      {
	to[0]= from[3];
	to[1]= from[2];
	to[2]= from[1];
	to[3]= from[0];
	from+= 4;
	to+= 4;
      }
    firstWord+= scanLine;
    lastWord+= scanLine;
  }
}




int clipboardSize(void)
{
#ifndef HEADLESS
  return 0;
#else
  return 0;
#endif
}

/* claim ownership of the X selection, providing the given string to requestors */
int clipboardWriteFromAt(int count, int byteArrayIndex, int startIndex)
{
#ifndef HEADLESS
#endif
}

/* transfer the X selection into the given byte array; optimise local requests */
int clipboardReadIntoAt(int count, int byteArrayIndex, int startIndex)
{
#ifndef HEADLESS
  return 0;
#else
  return 0;
#endif
}


/*** Access to system attributes and command-line arguments ***/

static char *getSpecialAttribute(int id)
{
  static char buf[256];
  switch (id) 
  	{
  	case 0:	
    	sprintf(buf, "%s", shortImageName);
    	return buf;
	default:
		sprintf(buf, "");
  		return buf;
  	}
}

int attributeSize(int id)
{
  if (id + 1 >= initialArgc) return 0;
  if (id < 0)
    return strlen(getSpecialAttribute(-id));
  else
    return strlen(initialArgv[id + 1]);
}

int getAttributeIntoLength(int id, int byteArrayIndex, int length)
{
  char *attrIndex= (char *)byteArrayIndex;
  char *arg;
  if (id + 1>= initialArgc) return 0;
  if (id < 0)
    arg= getSpecialAttribute(-id);
  else
    arg= initialArgv[id + 1];
  while (length--)
    *attrIndex++= *arg++;
  return 0;
}



static char *progName;
void usage()
	{
  	printf("Usage: %s [<options>] [<imageName>]\n\n", progName);
  	printf("<options> are:\n");
  	printf("  -res <widthxheight>  display resolution (default: AutoDetect)\n");
  	printf("  -version             print version information, then exit\n");
  	printf("\nNotes:\n");
 	printf("  <imageName> defaults to 'squeak.ima'.\n");
  	exit(1);
}



void imageNotFound(char *imageName)
{
  /* image file is not found */
  fprintf(stderr, "Could not open the Squeak image file '%s'.\n\n", imageName);
  fprintf(stderr, "There are two ways to open a Squeak image file.  You can:\n");
  fprintf(stderr, "  1. Put the name of the image file on the command line\n");
  fprintf(stderr, "     when you run Squeak (use '-help' for more information).\n");
  fprintf(stderr, "  2. Save an image (from inside Squeak) with the name 'clone'\n");
  fprintf(stderr, "     and run Squeak from the directory containing it.\n");
  exit(1);
}


int strtobkm(char *str) {
  char *suffix;
  int value= strtol(str, &suffix, 10);
  switch (*suffix) {
  case 'k': case 'K':
    value*= 1024;
    break;
  case 'm': case 'M':
    value*= 1024*1024;
    break;
  }
  return value;
}


void ParseArguments(int argc, char **argv)
{
  char *ev;

  initialArgv= argv;
  initialArgc= 0;

  /* defaults */
  if (ev= getenv("SQUEAK_IMAGE"))
    strcpy(shortImageName, ev);
  else
    strcpy(shortImageName, "squeak.ima");

  if (ev= getenv("SQUEAK_MEMORY"))
    initialHeapSize= strtobkm(ev);
  else
    initialHeapSize= 5*1024*1024;	/* 5 megabytes by default */

  argc--;
  argv++;	/* skip VM name */

  while (argc && **argv == '-')	/* more options to parse */
    {
      if (!strcmp(*argv, "-help"))
		usage();
	  argc--;	/* skip -display */
	  argv++;
	  if (!argc) usage();
	  displayName= *argv++;
	  argc--;
	  continue;

	if (!strcmp(*argv, "-memory"))
		{
	  	argc--;	/* skip -memory */
	  	argv++;
	  	if (!argc) usage();
	  		initialHeapSize= strtobkm(*argv++);
	  	argc--;
	  	continue;
		}

      if (!strcmp(*argv, "-version"))
		{
	  	extern int vm_serial;
	  	extern char *vm_date, *vm_version, *cc_version, *ux_version;
	  	extern char *up_version;
	  	fprintf(stderr, "%s/%d #%d %s %s\n%s\n%s\n",
		  vm_version, CURRENT_VERSION, vm_serial,
		  vm_date, cc_version, ux_version, up_version);
	  	exit(0);
		}

      /* assume the argument is meant for Squeak, and silently ignore it */
      initialArgv[initialArgc++]= *argv++;
      --argc;
    }

  if (argc)
    {
      strcpy(shortImageName, *argv);
      initialArgv[initialArgc++]= *argv++;
      --argc;
    }
  initialArgv[initialArgc]= (char *)0;
}



/*** Release ***/
void Cleanup(void)
{
  /* not all of these are essential, but they're polite... */
  /* if (stColormap) XFreeColormap(stDisplay, stColormap);
  if (stGC)	  XFreeGC(stDisplay, stGC);
  if (stWindow)	  XDestroyWindow(stDisplay, stWindow);
  XCloseDisplay(stDisplay); */
}


/*** Image File Naming ***/
int imageNameSize(void)
{
  return strlen(imageName);
}


int imageNameGetLength(int sqImageNameIndex, int length)
{
  char *sqImageName= (char *)sqImageNameIndex;
  int count, i;

  count= strlen(imageName);
  count= (length < count) ? length : count;

  /* copy the file name into the Squeak string */
  for (i= 0; i < count; i++)
    sqImageName[i]= imageName[i];

  return count;
}


int imageNamePutLength(int sqImageNameIndex, int length)
{
  char *sqImageName= (char *)sqImageNameIndex;
  int count, i;

  count= (IMAGE_NAME_SIZE < length) ? IMAGE_NAME_SIZE : length;

  /* copy the file name into a null-terminated C string */
  for (i= 0; i < count; i++)
    imageName[i]= sqImageName[i];

  imageName[count]= 0;

  return count;
}


/*** Profiling ***/

int clearProfile(void) {}
int dumpProfile(void) {}
int startProfiling(void) {}
int stopProfiling(void) {}


/*** Timing support ***/

void SetUpTimers(void)
{
  gettimeofday(&startUpTime, 0);
}



void printByte (unsigned int val)
	{
	printf("%d%d%d%d%d%d%d%d ", 
			(val & 128) == 128,
			(val & 64) == 64 , 
			(val & 32) == 32, 
			(val & 16) == 16, 
			(val & 8) == 8, 
			(val & 4) == 4, 
			(val & 2) == 2, 
			(val & 1) == 1);
	}



void RecordFullPathForImageName(char *localImageName)
{
  int i;
  /* get the full path for the image */
  getcwd (vmPath, MAXPATHLEN);
  strcat(vmPath,"\\");
  for (i= strlen(vmPath); i >= 0; i--)
  	{
    if ('/' == vmPath[i])
      vmPath[i]= '\\';
    }
  strcpy (imageName, vmPath);
  strcat (imageName, shortImageName);
}



setupFilesAndPath(int argc, char **argv)

	{
  	/* re-open stdout && stderr */
	outlog = freopen("outlog.txt", "a+", stdout);
	errlog = freopen("errlog.txt", "a+", stderr);
	}



void main (int argc, char **argv) {
	/* initialization */

	int card;
	progName= argv[0];

	setupFilesAndPath(argc, argv);
   	ParseArguments(argc, argv);
  	SetUpTimers();
	SetUpWindow(displayName);
	#ifdef NEED_TZSET
  		tzset();	/* should _not_ be necessary! */
	#endif
  	RecordFullPathForImageName(shortImageName);
//printf ("3 Image name is %s\n", imageName);
//printf ("3 Short image name is %s\n", shortImageName);
//printf ("3 VmPath is %s \n", vmPath);
//scanf ("%c");

	/* check the interpreter's size assumptions for basic data types */
     if (sizeof(int) != 4) error("This C compiler's integers are not 32 bits.");
     if (sizeof(double) != 8) error("This C compiler's doubles are not 64 bits.");
     if (sizeof(time_t) != 4) error("This C compiler's time_t's are not 32 bits.");
	
	allegro_init();
    install_keyboard();
    install_timer();
    install_mouse();

    /* read the image file and allocate memory for Squeak heap */
    {
    FILE *f= fopen(shortImageName, "rb");
    if (f == NULL)
      imageNotFound(shortImageName);
    readImageFromFileHeapSize(f, initialHeapSize);
    fclose(f);
	}

	/* Prompt the user for the correct driver/screen size */
	set_gfx_mode(GFX_VGA, 320, 200, 0, 0);
      set_pallete(desktop_pallete);		
	if (!gfx_mode_select(&card, &stWidth, &stHeight)) 
		{
      	allegro_exit();
      	exit(1);
   		}
    if (set_gfx_mode (card,	stWidth, stHeight, 0, 0) != 0)
    	{
        printf ("Error setting selected graphics mode\nTry 320x200x256 or 640x400x256 mode\n");
        exit (1);
        }
      /* open the Squeak window */
      SetWindowSize();
      /* make a memory bitmap sized widthxheight */
      memory_bitmap = create_bitmap(stWidth, stHeight);
      clear(memory_bitmap);
      SetUpPixmaps();
      current_cursor = create_bitmap(16, 16);
      set_mouse_speed (0,0);

     /* run Squeak */
     interpret();
     destroy_bitmap(memory_bitmap);
}

