#include "allegro.h"

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>

#ifndef SQ_IMAGE_VERSION_MAJOR
#define SQ_IMAGE_VERSION_MAJOR 1
#endif
#ifndef SQ_IMAGE_VERSION_MINOR
#define SQ_IMAGE_VERSION_MINOR 20
#endif

/* before 1.19 we don't have network primitives */
#if ((SQ_IMAGE_VERSION_MAJOR == 1) && (SQ_IMAGE_VERSION_MINOR < 19))
#define NO_NETWORKING
#endif

/* before 1.20 we look for sources and changes */
#if ((SQ_IMAGE_VERSION_MAJOR == 1) && (SQ_IMAGE_VERSION_MINOR < 19))
#define VM_MANUAL_CHECK_FOR_FILES
#endif


#if defined(_M_IX86) && !defined(DOUBLE_WORD_ALIGNMENT)
/* define DOUBLE_WORD_ALIGNMENT for X86 machines */
#define DOUBLE_WORD_ALIGNMENT
#elif !defined(DOUBLE_WORD_ALIGNMENT)
#pragma message("Sure you don't want double word alignment?!")
#endif

#define DOUBLE_WORD_ALIGNMENT


/* platform-dependent float conversion macros */
/* Note: Second argument must be a variable name, not an expression! */
/* Note: Floats in image are always in PowerPC word order; change
   these macros to swap words if necessary. This costs no extra and
   obviates sometimes having to word-swap floats when reading an image.
*/
#ifdef DOUBLE_WORD_ALIGNMENT
/* word-based copy for machines that require doubles to be double-word aligned */
#define storeFloatAtfrom(i, floatVarName) \
	*((int *) (i) + 0) = *((int *) &(floatVarName) + 1); \
	*((int *) (i) + 1) = *((int *) &(floatVarName) + 0);
#define fetchFloatAtinto(i, floatVarName) \
	*((int *) &(floatVarName) + 0) = *((int *) (i) + 1); \
	*((int *) &(floatVarName) + 1) = *((int *) (i) + 0);
#else
/* for machines that allow doubles to be on any word boundary */
#define storeFloatAtfrom(i, floatVarName) \
	*((double *) (i)) = (floatVarName);
#define fetchFloatAtinto(i, floatVarName) \
	(floatVarName) = *((double *) (i));
#endif

/*** increment this version number when the image file format changes ***/
#define CURRENT_VERSION 6502

#define true	1
#define false	0
#define null 0  

/* squeak file record; see sqFilePrims.c for details */
typedef struct {
	FILE	*file;
	int		sessionID;
	int		writable;
	int		fileSize;
	int		lastOp;  /* 0 = uncommitted, 1 = read, 2 = write */
} SQFile;

/* file i/o */
int sqFileAtEnd(SQFile *f);
int sqFileClose(SQFile *f);
int sqFileDeleteNameSize(int sqFileNameIndex, int sqFileNameSize);
int sqFileGetPosition(SQFile *f);
int sqFileInit(void);
int sqFileOpen(SQFile *f, int sqFileNameIndex, int sqFileNameSize, int writeFlag);
int sqFileReadIntoAt(SQFile *f, int count, int byteArrayIndex, int startIndex);
int sqFileRenameOldSizeNewSize(int oldNameIndex, int oldNameSize, int newNameIndex, int newNameSize);
int sqFileSetPosition(SQFile *f, int position);
int sqFileSize(SQFile *f);
int sqFileValid(SQFile *f);
int sqFileWriteFromAt(SQFile *f, int count, int byteArrayIndex, int startIndex);

/* directories */
int dir_Create(char *pathString, int pathStringLength);
int dir_Delimitor(void);
int dir_Lookup(char *pathString, int pathStringLength, int index,
	/* outputs: */
	char *name, int *nameLength, int *creationDate, int *modificationDate,
	int *isDirectory, int *sizeIfFile);
int dir_PathToWorkingDir(char *pathName, int pathNameMax);
int dir_SetMacFileTypeAndCreator(char *filename, int filenameSize, char *fType, char *fCreator);

/* interpreter entry points */
void error(char *s);
int checkedByteAt(int byteAddress);
int checkedByteAtput(int byteAddress, int byte);
int checkedLongAt(int byteAddress);
int checkedLongAtput(int byteAddress, int a32BitInteger);
int fullDisplayUpdate(void);
int initializeInterpreter(int bytesToShift);
int interpret(void);
int success(int);

/* display, mouse, keyboard, time i/o */
int ioBeep(void);
int ioGetButtonState(void);
int ioGetKeystroke(void);
int ioMicroMSecs(void);
int ioMousePoint(void);
int ioPeekKeystroke(void);
int ioProcessEvents(void);
int ioScreenSize(void);
int ioSeconds(void);
int ioSetCursor(int cursorBitsIndex, int offsetX, int offsetY);
int ioShowDisplay(
	int dispBitsIndex, int width, int height, int depth,
	int affectedL, int affectedR, int affectedT, int affectedB);

/* millisecond clock based on vertical retrace timer (1/60th second) */
extern int ioMSecs();
/*#define ioMSecs() ((1000 * clock()) / CLOCKS_PER_SEC)*/

/* event capture */
/* int recordMouseDown(XButtonEvent *theEvent);
int recordModifierButtons(XButtonEvent *theEvent);
int recordKeystroke(XKeyEvent *theEvent); */

/* image file and VM path names */
extern char imageName[];
int imageNameGetLength(int sqImageNameIndex, int length);
int imageNamePutLength(int sqImageNameIndex, int length);
int imageNameSize(void);
int vmPathSize(void);
int vmPathGetLength(int sqVMPathIndex, int length);

/* save/restore */
int readImageFile(char *imageFilename);
int writeImageFile(int imageBytes);

/* clipboard (cut/copy/paste) */
int clipboardSize(void);
int clipboardReadIntoAt(int count, int byteArrayIndex, int startIndex);
int clipboardWriteFromAt(int count, int byteArrayIndex, int startIndex);

/*******************************************************************/
/*  Sound output functions                                         */
/*******************************************************************/

int snd_AvailableSpace(void);
int snd_PlaySamplesFromAtLength(int frameCount, int arrayIndex, int startIndex);
int snd_PlaySilence(void);
int snd_Start(int frameCount, int samplesPerSec, int stereo, int semaphoreIndex);
int snd_Stop(void);
int snd_InsertSamplesFromLeadTime(int frameCount, int srcBufPtr, int samplesOfLeadTime);

/*******************************************************************/
/*  Sound input functions                                          */
/*******************************************************************/

int snd_SetRecordLevel(int level);
int snd_StartRecording(int desiredSamplesPerSec, int stereo, int semaphoreIndex);
int snd_StopRecording(void);
double snd_GetRecordingSampleRate(void);
int snd_RecordSamplesIntoAtLength(int buf, int startSliceIndex, int bufferSizeInBytes);

/* sound generation primitives */
int primWaveTableSoundmixSampleCountintostartingAtpan(void);
int primFMSoundmixSampleCountintostartingAtpan(void);
int primPluckedSoundmixSampleCountintostartingAtpan(void);
int primSampledSoundmixSampleCountintostartingAtpan(void);


/* joystick support */
int joystickInit(void);
int joystickRead(int stickIndex);

/* netscape plug-in support */
int plugInInit(char *imageName);
int plugInShutdown(void);
int plugInInterpretCycles(int cycleCount);

/* interpreter entry points needed by compiled primitives */
void * arrayValueOf(int arrayOop);
int checkedIntegerValueOf(int intOop);
void * fetchArrayofObject(int fieldIndex, int objectPointer);
double fetchFloatofObject(int fieldIndex, int objectPointer);
int fetchIntegerofObject(int fieldIndex, int objectPointer);
double floatValueOf(int floatOop);
int pop(int nItems);
int storeIntegerofObjectwithValue(int fieldIndex, int objectPointer, int integerValue);

/* sound generation primitives */
int primWaveTableSoundmixSampleCountintostartingAtpan(void);
int primFMSoundmixSampleCountintostartingAtpan(void);
int primPluckedSoundmixSampleCountintostartingAtpan(void);

/* squeak socket record; see sqMacNetwork.c for details */
typedef struct {
	int		sessionID;
	int		socketType;  /* 0 = TCP, 1 = UDP */
	void	*privateSocketPtr;
}  SQSocket, *SocketPtr;

/* networking primitives */
int	sqNetworkInit(int resolverSemaIndex);
void	sqNetworkShutdown(void);
void	sqResolverAbort(void);
void	sqResolverAddrLookupResult(char *nameForAddress, int nameSize);
int	sqResolverAddrLookupResultSize(void);
int	sqResolverError(void);
int	sqResolverLocalAddress(void);
int	sqResolverNameLookupResult(void);
void	sqResolverStartAddrLookup(int address);
void	sqResolverStartNameLookup(char *hostName, int nameSize);
int	sqResolverStatus(void);
void	sqSocketAbortConnection(SocketPtr s);
void	sqSocketCloseConnection(SocketPtr s);
int	sqSocketConnectionStatus(SocketPtr s);
void	sqSocketConnectToPort(SocketPtr s, int addr, int port);
void	sqSocketCreateNetTypeSocketTypeRecvBytesSendBytesSemaID(
	SocketPtr s, int netType, int socketType,
	int recvBufSize, int sendBufSize, int semaIndex);
void	sqSocketDestroy(SocketPtr s);
int	sqSocketError(SocketPtr s);
void	sqSocketListenOnPort(SocketPtr s, int port);
int	sqSocketLocalAddress(SocketPtr s);
int	sqSocketLocalPort(SocketPtr s);
int	sqSocketReceiveDataAvailable(SocketPtr s);
int	sqSocketReceiveDataBufCount(SocketPtr s, int buf, int bufSize);
int	sqSocketRemoteAddress(SocketPtr s);
int	sqSocketRemotePort(SocketPtr s);
int	sqSocketSendDataBufCount(SocketPtr s, int buf, int bufSize);
int	sqSocketSendDone(SocketPtr s);



/* profiling */
FILE * outlog;
FILE * errlog;
int clearProfile(void);
int dumpProfile(void);
int startProfiling(void);
int stopProfiling(void);
